@@ -147,6 +147,17 @@ class MapWidget(gtk.Widget):
     # check if redrawing is enabled
     if(self.d.get("needRedraw", False)):
       self.forceRedraw()
+      
+  def forceRedraw(self):
+    """Make the window trigger a draw event.
+    TODO: consider replacing this if porting pyroute to another platform"""
+    self.d['needRedraw'] = False
+    if self.redraw:
+      try:
+        self.window.invalidate_rect((0,0,self.rect.width,self.rect.height),False)
+      except Exception, e:
+        print "error in screen invalidating function"
+        print "exception: %s" % e
 
   def mousedown(self,x,y):
     """this signalizes start of a drag or a just a click"""
@@ -180,6 +191,13 @@ class MapWidget(gtk.Widget):
       if self.centeringDisableTreshold:
         if distSq > self.centeringDisableTreshold:
           self.d["centred"] = False # turn off centering after dragging the map (like in TangoGPS)
+          m = self.m.get('messages', None)
+          if m:
+            """the device module might have to update button state if it has an
+            external application menu with toggle buttons
+            eq. if centering state changes by dragging the map, it needs to update its
+            toggle button state"""
+            m.sendMessage('device:updateAppMenu')
           self.d["needRedraw"] = True
     else:
       if self.altMapDragEnabled:
@@ -215,17 +233,6 @@ class MapWidget(gtk.Widget):
     """
     self.centeringDisableTreshold = treshold
 
-  def forceRedraw(self):
-    """Make the window trigger a draw event.  
-    TODO: consider replacing this if porting pyroute to another platform"""
-    self.d['needRedraw'] = False
-    if self.redraw:
-      try:
-        self.window.invalidate_rect((0,0,self.rect.width,self.rect.height),False)
-      except Exception, e:
-        print "error in screen invalidating function"
-        print "exception: %s" % e
-
   def draw(self, cr, event):
     """ re/Draw the modrana GUI """
     start = time.clock()
@@ -583,7 +590,22 @@ class GuiBase:
     self.addTime("GUI creation")
 
     # Create the window
-    win = gtk.Window()
+
+    #TODO: do this more cleanly:
+
+    """when on N900, use a hildon StackableWindow, which
+    enables app menu and tohe features on Maemo 5"""
+    if device == 'n900':
+      try:
+        import hildon
+        win = hildon.StackableWindow()
+      except Exception, e:
+        print "creating hildon stackable window failed"
+        print e
+        win = gtk.Window()
+    else:
+      win = gtk.Window()
+
     win.set_title('modRana')
     win.connect('delete-event', gtk.main_quit)
     self.addTime("window created")
@@ -624,7 +646,7 @@ class GuiBase:
     event_box.connect("button_release_event", self.released)
     event_box.connect("motion_notify_event", self.moved)
     win.add(event_box)
-    
+
     # Create the map
     self.mapWidget = MapWidget()
     self.mapWidget.topWindow=win # make the main widown accessible from modules
