@@ -42,7 +42,12 @@ class ranaModule:
     if(self.module_exists('watchlist')):
       self.m['watchlist'].notify(name, value, self.d.get(name, None))
     self.d[name] = value
-    
+
+  def notify(self, message, msTimeout=0, icon=""):
+    if self.dmod: # if some module sends a notification during init, the device module might not be loaded
+      self.dmod.notify(message, msTimeout=0, icon="")
+      # TODO: notification module fallback
+
   def getStatus(self):
     return(self.status)
   
