@@ -79,6 +79,37 @@ class deviceModule(ranaModule):
        (enbale fullscreen, etc.)"""
     pass
 
+  def hasNativeNotificationSupport(self):
+    """report if the device provides its own notification method"""
+    return False
+
+  def notify(self, message, msTimeout=0, icon=""):
+    """send a notification"""
+    pass
+
+  def hasKeyboard(self):
+    """report if the device has a keyboard"""
+    return True
+
+  def hasButtons(self):
+    """report if the device has some usable buttons other than keyboard"""
+    if self.hasVolumeKeys():
+      return True
+    else:
+      return False
+
+  def hasVolumeKeys(self):
+    """report if the device has application-usable volume control keys or their
+    equivalent - basically just two nearby button that can be used for zooming up/down,
+    skipping to next/last and similar actions"""
+    return False
+
+  def enableVolumeKeys(self):
+    pass
+
+  def enableVolumeKeys(self):
+    pass
+
   
 #  def getAutorotationSupported(self):
 #    return False
