@@ -20,15 +20,14 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from base_device_module import deviceModule
-import time
 import dbus.glib
+import hildon
+import gtk
 """
 why dbus.glib ?
 if you import only "dbus", it can't find its mainloop for callbacks
 
 """
-#from dbus.mainloop.glib import DBusGMainLoop
-
 
 def getModule(m,d,i):
   return(device_n900(m,d,i))
@@ -53,6 +52,19 @@ class device_n900(deviceModule):
     self.mceSignalInterface.connect_to_signal("display_status_ind", self.screenStateChangedCallback)
     print "N900: dbus initialized"
 
+    # app menu and buttons
+    self.centeringToggleButton = None
+    self.rotationToggleButton = None
+    self.soundToggleButton = None
+    self._addHildonAppMenu()
+
+    # enable volume keys usage
+    if self.get('useVolumeKeys', True):
+      self._updateVolumeKeys()
+
+
+    print "N900: application menu added"
+
     print "N900 device specific module initialized"
 
   def firstTime(self):
@@ -78,7 +90,6 @@ class device_n900(deviceModule):
       if self.topWindow: #TODO: do this more efficiently
         self.startAutorotation()
         self.done = True
-    
 
   def handleMessage(self, message, type, args):
     if message == 'modeChanged':
@@ -86,7 +97,11 @@ class device_n900(deviceModule):
       if rotationMode:
         self.setRotationMode(rotationMode)
         print "rotation mode changed"
-
+    elif message == 'updateAppMenu':
+      self._updateAppMenu()
+    elif message == 'updateKeys':
+      self._updateVolumeKeys()
+      
   def getDeviceName(self):
     return "Nokia N900"
 
@@ -178,6 +193,103 @@ class device_n900(deviceModule):
       elif state== "off":
         display.disableRedraw(reason="N900 display blanked")
 
+  def hasNativeNotificationSupport(self):
+    return True
+
+  def notify(self, message, msTimeout=0, icon="icon_text"):
+    """the third barameter has to be a non zerolength string or
+    else the banner is not created"""
+    #TODO: find what strings to submit to actually get an icon displayed
+
+    if len(icon) == 0:
+      icon = "spam" # as mentioned above, the string has to be longer tahn zero
+
+    banner = hildon.hildon_banner_show_information_with_markup(self.modrana.topWindow, icon, message)
+    if msTimeout:
+      banner.set_timeout(msTimeout)
+
+  def hasButtons(self):
+    """the N900 has the volume keys (2 buttons), the camerra trigger (2 states)
+    and the proximity sensor,
+    other than that state of the camera cover and kyboard slider can be sensed
+    AND there is the accelerometer and light sensor :)
+    """
+    return True
+
+  def hasVolumeKeys(self):
+    return True
+
+  def enableVolumeKeys(self):
+    if self.modrana.topWindow.flags() & gtk.REALIZED:
+      self.enable_volume_cb()
+    else:
+      self.modrana.topWindow.connect("realize", self.enable_volume_cb)
+
+  def disableVolumeKeys(self):
+    self.modrana.topWindow.window.property_change(gtk.gdk.atom_intern("_HILDON_ZOOM_KEY_ATOM"), gtk.gdk.atom_intern("INTEGER"), 32, gtk.gdk.PROP_MODE_REPLACE, [0]);
+
+  def enable_volume_cb(self, window=None):
+    self.modrana.topWindow.window.property_change(gtk.gdk.atom_intern("_HILDON_ZOOM_KEY_ATOM"), gtk.gdk.atom_intern("INTEGER"), 32, gtk.gdk.PROP_MODE_REPLACE, [1]);
+
+  def _updateVolumeKeys(self):
+    """check if volume keys should be used or not"""
+    if self.get('useVolumeKeys', True):
+      self.enableVolumeKeys()
+    else:
+      self.disableVolumeKeys()
+
+
+  def _addHildonAppMenu(self):
+    menu = hildon.AppMenu()
+    self.centeringToggleButton = gtk.ToggleButton(label="Centering")
+    self.centeringToggleButton.connect('toggled',self._toggle, 'centred')
+#    openFolderButton.connect('clicked',self.startFolderChooser)
+    self.rotationToggleButton = gtk.ToggleButton(label="Map rotation")
+    self.rotationToggleButton.connect('toggled',self._toggle,'rotateMap')
+    self.soundToggleButton = gtk.ToggleButton(label="Sound")
+    self.soundToggleButton.connect('toggled',self._toggle,'soundEnabled')
+
+    mapButton = gtk.Button("Map screen")
+    mapButton.connect('clicked',self._switchToMenu, None)
+    optionsButton = gtk.Button("Options")
+    optionsButton.connect('clicked',self._switchToMenu,'options')
+    searchButton = gtk.Button("Search")
+    searchButton.connect('clicked',self._switchToMenu,'search')
+    routeButton = gtk.Button("Route")
+    routeButton.connect('clicked',self._switchToMenu,'route')
+
+    self._updateAppMenu() # update initial button states
+
+    menu.append(self.centeringToggleButton)
+    menu.append(self.rotationToggleButton)
+    menu.append(self.soundToggleButton)
+    menu.append(mapButton)
+    menu.append(optionsButton)
+    menu.append(searchButton)
+    menu.append(routeButton)
+
+    # Show all menu items
+    menu.show_all()
+
+    # Add the menu to the window
+    self.modrana.topWindow.set_app_menu(menu)
+
+  def _toggle(self,toggleButton, key):
+    print "N900: key %s toggled" % key
+    self.set(key, toggleButton.get_active())
+
+  def _switchToMenu(self,toggleButton, menu):
+    """callback for the appMenu buttons, switch to a specified menu"""
+    self.set('menu', menu)
+    self.set('needRedraw', True)
+
+  def _updateAppMenu(self):
+    if self.centeringToggleButton:
+      self.centeringToggleButton.set_active(self.get("centred",True))
+    if self.rotationToggleButton:
+      self.rotationToggleButton.set_active(self.get("rotateMap",True))
+    if self.soundToggleButton:
+      self.soundToggleButton.set_active(self.get("soundEnabled",True))
 
 if(__name__ == "__main__"):
   a = n900({}, {})
