@@ -27,6 +27,7 @@ import cairo
 import urllib2
 import gtk
 import time
+import modrana_utils
 from configobj import ConfigObj
 from tilenames import *
 
@@ -96,6 +97,7 @@ class mapTiles(ranaModule):
     self.threadlListCondition = threading.Condition(threading.Lock())
     self.maxImagesInMemmory = 150 # to avoid a memmory leak
     self.imagesTrimmingAmmount = 30 # how many tiles to remove once the maximum is reached
+    self.loadRequestCStackSize = 10 # size for the circular loading request stack
     """so that trim does not run always run arter adding a tile"""
     self.tileSide = 256 # by default, the tiles are squares, side=256
     """ TODO: analyse memmory usage,
@@ -103,7 +105,8 @@ class mapTiles(ranaModule):
               platform dependendt value,
               user configurable
     """
-    self.tileLoadRequestQueue = Queue.Queue()
+    self.loadRequestCStack = modrana_utils.SynchronizedCircularStack(self.loadRequestCStackSize)
+    self.loadingNotifyQueue = Queue.Queue(1)
     self.downloadRequestPool = []
     self.downloadRequestPoolLock = threading.Lock()
     self.downloadRequestTimeout = 30 # in seconds
@@ -152,14 +155,14 @@ class mapTiles(ranaModule):
             break
           with self.downloadRequestPoolLock:
             activeThreads = len(self.threads)
-            maxThreads = int(self.get("maxAutoDownloadThreads", 20))
+            maxThreads = int(self.get("maxAutoDownloadThreads2", 10))
             if activeThreads < maxThreads: # can we start new threads ?
               # ** there are free download slots **
               availableSlots = maxThreads-activeThreads
               #fill all available slots
               for i in range(0, availableSlots):
                 if self.downloadRequestPool:
-                  request = self.downloadRequestPool.pop()
+                  request = self.downloadRequestPool.pop() # download most recent requests first
                   (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp) = request
                   # start a new thread
                   self.threads[name] = self.tileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, folder, self)
@@ -201,12 +204,17 @@ class mapTiles(ranaModule):
   def tileLoader(self):
     """this is a tile loading request consumer thread"""
     while True:
-      request = self.tileLoadRequestQueue.get(block=True) # consume from this queue
-      (key,args) = request
-      if key == 'loadRequest':
-        (name, x, y, z, layer) = args
-        self.loadImage(name, x, y, z, layer)
-      elif key == 'shutdown':
+      request = self.loadingNotifyQueue.get(block=True) # consume from this queue
+      if request == 'load':
+        # start processing loading requests from the stack
+        while(1):
+          (item,valid) = self.loadRequestCStack.popValid()
+          if valid:
+            (name, x, y, z, layer) = item
+            self.loadImage(name, x, y, z, layer)
+          else:
+            break # the stack is empty
+      elif request == 'shutdown':
         print "\nmapTiles: tile loading thread shutting down"
         break
 
@@ -273,7 +281,8 @@ class mapTiles(ranaModule):
     try: # this should get rid of a fair share of the infamous "black screens"
       proj = self.m.get('projection', None)
       if proj and proj.isValid():
-        loadingTile = self.loadingTile
+        loadingTileImageSurface = self.loadingTile[0]
+        requests = []
         (sx,sy,sw,sh) = self.get('viewport') # get screen parameters
         scale = int(self.get('mapScale', 1)) # get the current scale
 
@@ -326,100 +335,101 @@ class mapTiles(ranaModule):
           layer2 = self.get('layer2', 'mapnik')
 
           # draw the composited layer
-          for ix in range(0, wTiles):
-            for iy in range(0, hTiles):
+          with self.imagesLock: # just one lock per frame
+            for ix in range(0, wTiles):
+              for iy in range(0, hTiles):
 
-              # get tile cooridnates by incrementing the upper left tile cooridnates
-              x = cx+ix
-              y = cy+iy
+                # get tile cooridnates by incrementing the upper left tile cooridnates
+                x = cx+ix
+                y = cy+iy
 
-              # get screen coordinates by incrementing upper left tile screen coordinates
-              x1 = cx1 + 256*ix*scale
-              y1 = cy1 + 256*iy*scale
+                # get screen coordinates by incrementing upper left tile screen coordinates
+                x1 = cx1 + 256*ix*scale
+                y1 = cy1 + 256*iy*scale
 
-              # Try to load and display images
-              nameBack = "%s_%d_%d_%d" % (layer2,z,x,y)
-              nameOver = "%s_%d_%d_%d" % (layer,z,x,y)
-              backImage = self.images[0].get(nameBack)
-              overImage = self.images[0].get(nameOver)
-              # check if the background tile is already cached
-              if backImage == None: # background image not yet loaded
-                if self.imagesLock.acquire(False):
-                  self.images[0][nameBack] = loadingTile # set the loading tile as placeholder
-                  self.tileLoadRequestQueue.put(('loadRequest',(nameBack, x, y, z, layer2)),block=False)
-                  self.imagesLock.release()
-                backImage = loadingTile # draw the loading tile this time
-              # check if the overlay tile is already cached
-              if overImage == None: # overlay image not yet loaded
-                if self.imagesLock.acquire(False):
-                  self.images[0][nameOver] = loadingTile # set the loading tile as placeholder
-                  self.tileLoadRequestQueue.put(('loadRequest',(nameOver, x, y, z, layer)),block=False)
-                  self.imagesLock.release()
-                overImage = loadingTile # draw the loading tile this time
+                # Try to load and display images
+                nameBack = "%s_%d_%d_%d" % (layer2,z,x,y)
+                nameOver = "%s_%d_%d_%d" % (layer,z,x,y)
+                backImage = self.images[0].get(nameBack)
+                overImage = self.images[0].get(nameOver)
+                # check if the background tile is already cached
+                if backImage == None: # background image not yet loaded
+                  requests.append((nameBack, x, y, z, layer2))
+                  backImage = [loadingTileImageSurface] # draw the loading tile this time
 
-              """we do this inline to get rid of function calling overhead"""
-              # Move the cairo projection onto the area where we want to draw the image
-              cr.save()
-              cr.translate(x1,y1)
-              cr.scale(scale,scale) # scale te tile according to current scale settings
+                # check if the overlay tile is already cached
+                if overImage == None: # overlay image not yet loaded
+                  requests.append((nameOver, x, y, z, layer))
+                  overImage = [loadingTileImageSurface] # draw the loading tile this time
 
-              # Display the image
-              cr.set_source_surface(backImage[0],0,0) # draw the background
-              cr.paint_with_alpha(alphaBack)
-              cr.set_source_surface(overImage[0],0,0) # draw the overlay
-              cr.paint_with_alpha(alphaOver)
+                """we do this inline to get rid of function calling overhead"""
+                # Move the cairo projection onto the area where we want to draw the image
+                cr.save()
+                cr.translate(x1,y1)
+                cr.scale(scale,scale) # scale te tile according to current scale settings
 
-              # Return the cairo projection to what it was
-              cr.restore()
+                # Display the image
+                cr.set_source_surface(backImage[0],0,0) # draw the background
+                cr.paint_with_alpha(alphaBack)
+                cr.set_source_surface(overImage[0],0,0) # draw the overlay
+                cr.paint_with_alpha(alphaOver)
+
+                # Return the cairo projection to what it was
+                cr.restore()
 
         else: # overlay is disabled
-          # draw the normal layer
-          for ix in range(0, wTiles):
-            for iy in range(0, hTiles):
+          with self.imagesLock: # just one lock per frame
+            # draw the normal layer
+            for ix in range(0, wTiles):
+              for iy in range(0, hTiles):
 
-              # get tile cooridnates by incrementing the upper left tile cooridnates
-              x = cx+ix
-              y = cy+iy
+                # get tile cooridnates by incrementing the upper left tile cooridnates
+                x = cx+ix
+                y = cy+iy
 
-              # get screen coordinates by incrementing upper left tile screen coordinates
-              x1 = cx1 + 256*ix*scale
-              y1 = cy1 + 256*iy*scale
+                # get screen coordinates by incrementing upper left tile screen coordinates
+                x1 = cx1 + 256*ix*scale
+                y1 = cy1 + 256*iy*scale
 
-              # Try to load and display images
-              """we do this inline to get rid of function calling overhead"""
-              name = "%s_%d_%d_%d" % (layer,z,x,y)
-              tileImage = self.images[0].get(name)
-              if tileImage:
-                cr.save() # save the cairo projection context
-                cr.translate(x1,y1)
-                cr.scale(scale,scale)
-                cr.set_source_surface(tileImage[0],0,0)
-                cr.paint()
-                cr.restore() # Return the cairo projection to what it was
-              else:
-                """we need this lock to store the "work in progresss" temporary tile,
-                   which assures that there is only a single loading request queued for a tile
+                # Try to load and display images
+                """we do this inline to get rid of function calling overhead"""
+                name = "%s_%d_%d_%d" % (layer,z,x,y)
+                tileImage = self.images[0].get(name)
+                if tileImage:
+                  # tile found in memmory cache, draw it
+                  cr.save() # save the cairo projection context
+                  cr.translate(x1,y1)
+                  cr.scale(scale,scale)
+                  cr.set_source_surface(tileImage[0],0,0)
+                  cr.paint()
+                  cr.restore() # Return the cairo projection to what it was
+                else:       
+                  # tile not found in memmory cache, add a loading request
+                  requests.append((name, x, y, z, layer))
+                  cr.save() # save the cairo projection context
+                  cr.translate(x1,y1)
+                  cr.scale(scale,scale)
+                  cr.set_source_surface(loadingTileImageSurface,0,0)
+                  cr.paint()
+                  cr.restore() # Return the cairo projection to what it was
+        # batch send the requests to the loading thread
+        if requests:
+          self.loadRequestCStack.batchPush(requests)
+          """can the loadRequestCStack get full ?
+             NO :)
+             it takes the list, reverses it, extends its old internal
+             list with it - from this list a slice conforming to
+             its size limit is taken and set as the new internal list"""
+            # notify the loading thread using the notify Queue
+          self.loadingNotifyQueue.put("load", block=False)
 
-                   the lock is used quite often (trimming te image cache, storing new images),
-                   but we can't block in this thread as it would lag the user interface
-
-                   therefore if we need this lock, but it is unavailable,
-                   we just skip loading the "in progress tile" and quing the loading request
-                   and directly show a "loading..." tile instead
-                   """
-                if self.imagesLock.acquire(False):
-                  self.images[0][name] = loadingTile
-                  self.imagesLock.release()
-                  self.tileLoadRequestQueue.put(('loadRequest',(name, x, y, z, layer)),block=False)
-                cr.save() # save the cairo projection context
-                cr.translate(x1,y1)
-                cr.scale(scale,scale)
-                cr.set_source_surface(loadingTile[0],0,0)
-                cr.paint()
-                cr.restore() # Return the cairo projection to what it was
+    except Queue.Full:
+      """as we use the queue as a notification mechanism, we dont actually need to
+      process all the "load" notifications """
+      pass
 
     except Exception, e:
-      print "mapTiles: expception while drawing the map layer:\n%s" % e
+      print "mapTiles: expception while drawing the map layer: %s" % e
 
 
   def removeImageFromMemmory(self, name, dictIndex=0):
@@ -442,11 +452,6 @@ class mapTiles(ranaModule):
   def drawCompositeImage(self,cr, nameOver, nameBack, x,y, scale, alpha1=1, alpha2=1, dictIndex1=0,dictIndex2=0):
     """Draw a composited tile image"""
 
-#    # If it's not in memory, then stop here
-#    if not nameOver and nameBack in self.images.keys():
-#      print "Not loaded"
-#      return
-
     # Move the cairo projection onto the area where we want to draw the image
     cr.save()
     cr.translate(x,y)
@@ -570,7 +575,6 @@ class mapTiles(ranaModule):
       print "the tile image is corrupted nad/or there are no tiles for this zoomlevel, exception:\n%s" % e
       return False
 
-
   def storeInMemmory(self, surface, name, type="normal", expireTimestamp=None, dictIndex=0):
     """store a given image surface in the memmory image cache
        dictIndex = 0 -> nromal map tiles + tile specific error tiles
@@ -669,12 +673,15 @@ class mapTiles(ranaModule):
     self.shutdownAllThreads = True
 
     # shutdown the tile loading thread
-    self.tileLoadRequestQueue.put(('shutdown', ()),block=False)
+    try:
+      self.loadingNotifyQueue.put(('shutdown', ()),block=False)
+    except Queue.Full:
+      """the tile loading thread is demonic, so it will be still killed in the end"""
+      pass
     # notify the automatic tile download manager thread about the shutdown
     with self.threadlListCondition:
       self.threadlListCondition.notifyAll()
 
-
   class tileDownloader(Thread):
     """Downloads an image (in a thread)"""
     def __init__(self,name, x,y,z,layer,layerName,layerType,filename, folder, callback):
@@ -718,7 +725,7 @@ class mapTiles(ranaModule):
            after it is flushed with old tiles from the memmory
         """
       except urllib2.URLError, e:
-        tileNetworkErrorSurface = self.images[1]['tileNetworkError'][0]
+        tileNetworkErrorSurface = self.callback.images[1]['tileNetworkError'][0]
         expireTimestamp = time.time() + 10
         self.callback.storeInMemmory(tileNetworkErrorSurface,self.name, 'error', expireTimestamp) # retry after 10 seconds
         """ as not to DOS the system when we temorarily loose internet connection or other such error occurs,
@@ -736,7 +743,6 @@ class mapTiles(ranaModule):
         self.finished = 1 # finished thread can be removed from the set and retried
         self.removeSelf()
 
-
     def removeSelf(self):
         with self.callback.threadlListCondition:
           # try to remove its own instance from the thread list, so taht the instance could be garbage collected
@@ -762,14 +768,6 @@ class mapTiles(ranaModule):
 
     def downloadTile(self,name,x,y,z,layer,filename, folder):
       """Downloads an image"""
-#      layerDetails = maplayers.get(layer, None)
-    #  if(layerDetails == None):
-    #    return
-    #  if(layerDetails.get('pyrender',False)):
-    #    # Generate from local data
-    #    renderer = RenderModule.RenderClass()
-    #    renderer.RenderTile(z,x,y, 'default', filename) # TODO: pyrender layers
-    #  else:
       url = getTileUrl(x,y,z,layer)
 
       request = urllib2.urlopen(url)
@@ -780,10 +778,6 @@ class mapTiles(ranaModule):
 
       pl.write(content)
 
-#      if pl.write(content) == False:
-#        print "mapTiles:loading image failed"
-#        return
-
       pl.close() # this  blocks until the image is completely loaded
       # http://www.ossramblings.com/loading_jpg_into_cairo_surface_python
       #x = pixbuf.get_width()
