@@ -33,8 +33,6 @@ class options(ranaModule):
     self.on = '<span color="green">ON</span>'
     self.off = '<span color="red">OFF</span>'
   
-
-
   def addCategory(self,name,id,icon,actionPrefix="",actionSufix=""):
     """this metohod shuld be run only after menu module instance
     is vailable in self.menuModule and the options manu is clared,
@@ -71,7 +69,6 @@ class options(ranaModule):
       self.options[group][2].append(newOption)
     else:
       print "options: group %s does not exist, call addGroup to create it first" % group
-#      self.options[group] = [newOption,]
     
   def firstTime(self):
     """Create a load of options.  You can add your own options in here,
@@ -120,7 +117,9 @@ class options(ranaModule):
 
       # ** Rotation
       group = addGroup("Rotation", "map_rotation", catMap, "generic")
-      addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False)
+      addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False, 'device:updateAppMenu')
+      """the device module might show a device specific app-menu, so we have to notify it
+      about a chnage in the map rotation state"""
 
 
       # ** Scaling
@@ -184,6 +183,7 @@ class options(ranaModule):
                     ('sqlite',"sqlite (new, less space used)")],
                      group,
                      'files')
+      addBoolOpt("Store downloaded tiles", "storeDownloadedTiles", group, True)
 
     # * the view category *
     catView = addCat("View", "view", "view")
@@ -274,6 +274,17 @@ class options(ranaModule):
                    group,
                    4)
 
+
+    if self.dmod.hasButtons():
+      # TODO: change this once there are more options for key shortcuts
+      # * the Keys category *
+      catKeys = addCat("Keys", "keys", "keys")
+      # * Device buttons
+      group = addGroup("Device buttons", "device_buttons", catKeys, "generic")
+      if self.dmod.hasVolumeKeys():
+        addBoolOpt("Use volume keys for zooming", "useVolumeKeys", group, True, "device:updateKeys")
+
+
     # * the Navigation category
     catNavigation = addCat("Navigation", "navigation", "navigation")
 
@@ -415,16 +426,22 @@ class options(ranaModule):
        group,
        "full")
 
-    addOpt("Max nr. of threads for tile auto-download","maxAutoDownloadThreads",
+    addOpt("Max nr. of threads for tile auto-download","maxAutoDownloadThreads2",
       [(5, "5"),
-       (10, "10"),
-       (20, "20 (default)"),
+       (10, "10 (default)"),
+       (20, "20"),
        (30, "30"),
        (40, "40"),
        (50, "50")],
        group,
-       20)
+       10)
 
+    # * the Sound category *
+    catSound = addCat("Sound", "sound", "sound")
+    # * sound output
+    group = addGroup("Sound output", "sound_output", catSound, "generic")
+
+    addBoolOpt("Application wide sound output", "soundEnabled", group, True, "device:updateAppMenu")
 
 #    addOpt("Network", "threadedDownload",
 ##      [("off","No use of network"),
@@ -483,16 +500,6 @@ class options(ranaModule):
 #     False)
 
 
-#    # Add all our categories to the "options" menu
-#
-#    if(self.menuModule):
-#      for i in self.options.keys():
-#        self.menuModule.addItem(
-#          'options', # which menu to add to
-#          i, # title
-#          "opt_"+i, # icon name
-#          "set:menu:opt_%s|options:reset_scroll"%i ) # action
-
     # Set all undefined options to default values
     for category,options in self.options.items():
       for option in options[2]:
@@ -625,7 +632,6 @@ class options(ranaModule):
           onClick += "|options:save"
           onClick += "|set:needRedraw:1"
 
-#          y = y1 + (row+1) * dy
           y = y4 + (row) * dy
           w = w1 - (x4-x1)
           
